@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2001-2022 Free Software Foundation, Inc.
+  Copyright (C) 2001-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -33,9 +33,9 @@ struct system_ifconfig
 # define IF_VALID_TXQLEN 0x1
   int txqlen;
 };
-
 
 
+
 /* Output format support.  */
 
 # define _IU_CAT2(a,b) a ## b
@@ -112,7 +112,5 @@ _IU_EXTRN (rx_missed_errors)
 _IU_EXTRN (tx_aborted_errors)
 _IU_EXTRN (tx_carrier_errors)
 _IU_EXTRN (tx_fifo_errors)
-_IU_EXTRN (tx_heartbeat_errors)
-_IU_EXTRN (tx_window_errors)
-
+_IU_EXTRN (tx_heartbeat_errors) _IU_EXTRN (tx_window_errors)
 #endif
