@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free
-  Software Foundation, Inc.
+  Copyright (C) 2005-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -24,7 +23,7 @@
 #define USE_IPV6 1
 
 static PING *ping_init (int type, int ident);
-static int ping_set_dest (PING * ping, char *host);
+static int ping_set_dest (PING * ping, const char *host);
 static int ping_recv (PING * p);
 static int ping_xmit (PING * p);
 
