@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1994-2022 Free Software Foundation, Inc.
+  Copyright (C) 1994-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -160,16 +160,17 @@ char *LastArg;
 
 char **config_files;
 
-static bool env_option = false;	       /* Set environment variables */
-static bool resolve_option = false;    /* Resolve IP addresses */
-static bool pidfile_option = true;     /* Record the PID in a file */
+static bool env_option = false;	/* Set environment variables */
+static bool resolve_option = false;	/* Resolve IP addresses */
+static bool pidfile_option = true;	/* Record the PID in a file */
 static const char *pid_file = PATH_INETDPID;
 
 const char args_doc[] = "[CONF-FILE [CONF-DIR]]...";
 const char doc[] = "Internet super-server.";
 
 /* Define keys for long options that do not have short counterparts. */
-enum {
+enum
+{
   OPT_ENVIRON = 256,
   OPT_RESOLVE
 };
@@ -178,29 +179,30 @@ const char *program_authors[] = {
   "Alain Magloire", "Alfred M. Szmidt", "Debarshi Ray",
   "Jakob 'sparky' Kaivo", "Jeff Bailey",
   "Jeroen Dekkers", "Marcus Brinkmann", "Sergey Poznyakoff",
-  "others", NULL };
+  "others", NULL
+};
 
 static struct argp_option argp_options[] = {
 #define GRP 0
   {"debug", 'd', NULL, 0,
-   "turn on debugging, run in foreground mode", GRP+1},
+   "turn on debugging, run in foreground mode", GRP + 1},
   {"environment", OPT_ENVIRON, NULL, 0,
-   "pass local and remote socket information in environment variables", GRP+1},
-  { "pidfile", 'p', "PIDFILE", OPTION_ARG_OPTIONAL,
-    "override pidfile (default: \"" PATH_INETDPID "\")",
-    GRP+1 },
+   "pass local and remote socket information in environment variables",
+   GRP + 1},
+  {"pidfile", 'p', "PIDFILE", OPTION_ARG_OPTIONAL,
+   "override pidfile (default: \"" PATH_INETDPID "\")",
+   GRP + 1},
   {"rate", 'R', "NUMBER", 0,
-   "maximum invocation rate (per minute)", GRP+1},
+   "maximum invocation rate (per minute)", GRP + 1},
   {"resolve", OPT_RESOLVE, NULL, 0,
    "resolve IP addresses when setting environment variables "
-   "(see --environment)", GRP+1},
+   "(see --environment)", GRP + 1},
 #undef GRP
   {NULL, 0, NULL, 0, NULL, 0}
 };
 
 static error_t
-parse_opt (int key, char *arg,
-	   struct argp_state *state MAYBE_UNUSED)
+parse_opt (int key, char *arg, struct argp_state *state MAYBE_UNUSED)
 {
   char *p;
   int number;
@@ -226,9 +228,9 @@ parse_opt (int key, char *arg,
     case 'R':
       number = strtol (arg, &p, 0);
       if (number < 1 || *p)
-        syslog (LOG_ERR, "-R %s: bad value for service invocation rate", arg);
+	syslog (LOG_ERR, "-R %s: bad value for service invocation rate", arg);
       else
-        toomany = number;
+	toomany = number;
       break;
 
     case OPT_RESOLVE:
@@ -243,19 +245,19 @@ parse_opt (int key, char *arg,
 }
 
 static struct argp argp =
-  {argp_options, parse_opt, args_doc, doc, NULL, NULL, NULL};
-
+  { argp_options, parse_opt, args_doc, doc, NULL, NULL, NULL };
 
+
 struct servtab
 {
   const char *se_file;
   int se_line;
-  char *se_node;                /* node name */
+  char *se_node;		/* node name */
   char *se_service;		/* name of service */
   int se_socktype;		/* type of socket to use */
   char *se_proto;		/* protocol used */
   pid_t se_wait;		/* single threaded server */
-  unsigned se_max;              /* Maximum number of instances per CNT_INTVL */
+  unsigned se_max;		/* Maximum number of instances per CNT_INTVL */
   short se_checked;		/* looked at during merge */
   char *se_user;		/* user name to run as */
   char *se_group;		/* group name to run as */
@@ -270,7 +272,7 @@ struct servtab
   struct sockaddr_storage se_ctrladdr;	/* bound address */
   socklen_t se_addrlen;		/* exact address length in use */
   unsigned se_refcnt;
-  unsigned se_count;			/* number started since se_time */
+  unsigned se_count;		/* number started since se_time */
   struct timeval se_time;	/* start of se_count */
   struct servtab *se_next;
 } *servtab;
@@ -281,8 +283,8 @@ struct servtab
 #define ISMUX(sep)	(((sep)->se_type == MUX_TYPE) ||	\
 			 ((sep)->se_type == MUXPLUS_TYPE))
 #define ISMUXPLUS(sep)	((sep)->se_type == MUXPLUS_TYPE)
-
 
+
 /* Built-in services */
 void chargen_dg (int, struct servtab *);
 void chargen_stream (int, struct servtab *);
@@ -303,26 +305,25 @@ struct biltin
   short bi_fork;		/* 1 if should fork before call */
   short bi_wait;		/* 1 if should wait for child */
   void (*bi_fn) (int s, struct servtab *);	/*function which performs it */
-} biltins[] =
-  {
-    /* Echo received data */
-    {"echo", SOCK_STREAM, 1, 0, echo_stream},
-    {"echo", SOCK_DGRAM, 0, 0, echo_dg},
-    /* Internet /dev/null */
-    {"discard", SOCK_STREAM, 1, 0, discard_stream},
-    {"discard", SOCK_DGRAM, 0, 0, discard_dg},
-    /* Return 32 bit time since 1900 */
-    {"time", SOCK_STREAM, 0, 0, machtime_stream},
-    {"time", SOCK_DGRAM, 0, 0, machtime_dg},
-    /* Return human-readable time */
-    {"daytime", SOCK_STREAM, 0, 0, daytime_stream},
-    {"daytime", SOCK_DGRAM, 0, 0, daytime_dg},
-    /* Familiar character generator */
-    {"chargen", SOCK_STREAM, 1, 0, chargen_stream},
-    {"chargen", SOCK_DGRAM, 0, 0, chargen_dg},
-    {"tcpmux", SOCK_STREAM, 1, 0, tcpmux},
-    {NULL, 0, 0, 0, NULL}
-  };
+} biltins[] = {
+  /* Echo received data */
+  {"echo", SOCK_STREAM, 1, 0, echo_stream},
+  {"echo", SOCK_DGRAM, 0, 0, echo_dg},
+  /* Internet /dev/null */
+  {"discard", SOCK_STREAM, 1, 0, discard_stream},
+  {"discard", SOCK_DGRAM, 0, 0, discard_dg},
+  /* Return 32 bit time since 1900 */
+  {"time", SOCK_STREAM, 0, 0, machtime_stream},
+  {"time", SOCK_DGRAM, 0, 0, machtime_dg},
+  /* Return human-readable time */
+  {"daytime", SOCK_STREAM, 0, 0, daytime_stream},
+  {"daytime", SOCK_DGRAM, 0, 0, daytime_dg},
+  /* Familiar character generator */
+  {"chargen", SOCK_STREAM, 1, 0, chargen_stream},
+  {"chargen", SOCK_DGRAM, 0, 0, chargen_dg},
+  {"tcpmux", SOCK_STREAM, 1, 0, tcpmux},
+  {NULL, 0, 0, 0, NULL}
+};
 
 #define NUMINT	(sizeof(intab) / sizeof(struct inent))
 
@@ -337,8 +338,8 @@ bi_lookup (const struct servtab *sep)
       return bi;
   return NULL;
 }
-
 
+
 /* Signal handling */
 
 #if defined HAVE_SIGACTION
@@ -376,7 +377,7 @@ signal_set_handler (int signo, void (*handler) (int))
 }
 
 void
-signal_block (SIGSTATUS * old_status)
+signal_block (SIGSTATUS *old_status)
 {
 #ifdef HAVE_SIGACTION
   sigset_t sigs;
@@ -394,7 +395,7 @@ signal_block (SIGSTATUS * old_status)
 }
 
 void
-signal_unblock (SIGSTATUS * status)
+signal_unblock (SIGSTATUS *status)
 {
 #ifdef HAVE_SIGACTION
   if (status)
@@ -519,9 +520,9 @@ reapchild (int signo MAYBE_UNUSED)
 	  }
     }
 }
-
 
 
+
 char *
 newstr (const char *cp)
 {
@@ -550,10 +551,10 @@ void
 dupstr (char **pstr)
 {
   if (*pstr)
-    dupmem ((void**)pstr, strlen (*pstr) + 1);
+    dupmem ((void **) pstr, strlen (*pstr) + 1);
 }
-
 
+
 /*
  * print_service:
  *	Dump relevant information to stderr
@@ -567,15 +568,14 @@ print_service (const char *action, struct servtab *sep)
 	   sep->se_file, sep->se_line,
 	   action,
 	   ISMUX (sep) ? (ISMUXPLUS (sep) ? "tcpmuxplus" : "tcpmux")
-		      : (sep->se_node ? sep->se_node : "*"),
+	   : (sep->se_node ? sep->se_node : "*"),
 	   sep->se_service, sep->se_proto,
 	   (int) sep->se_wait, sep->se_max,
 	   sep->se_user, sep->se_group,
-	   sep->se_bi ? sep->se_bi->bi_service : "no",
-	   sep->se_server);
+	   sep->se_bi ? sep->se_bi->bi_service : "no", sep->se_server);
 }
-
 
+
 /* Configuration */
 int
 setup (struct servtab *sep)
@@ -583,12 +583,12 @@ setup (struct servtab *sep)
   int err;
   int on = 1;
 
- tryagain:
+tryagain:
   sep->se_fd = socket (sep->se_family, sep->se_socktype, 0);
   if (sep->se_fd < 0)
     {
       /* If we don't support creating AF_INET6 sockets, create AF_INET
-	 sockets.  */
+         sockets.  */
       if (errno == EAFNOSUPPORT && sep->se_family == AF_INET6
 	  && sep->se_v4mapped)
 	{
@@ -600,17 +600,16 @@ setup (struct servtab *sep)
       if (debug)
 	fprintf (stderr, "socket failed on %s/%s: %s\n",
 		 sep->se_service, sep->se_proto, strerror (errno));
-      syslog (LOG_ERR, "%s/%s: socket: %m",
-	      sep->se_service, sep->se_proto);
+      syslog (LOG_ERR, "%s/%s: socket: %m", sep->se_service, sep->se_proto);
       return 1;
     }
 #ifdef IPV6
   if (sep->se_family == AF_INET6)
     {
       /* Reverse the value of SEP->se_v4mapped, since otherwise if
-	 using `tcp6' as a protocol type, all connections will be
-	 mapped to IPv6, and with `tcp6only', IPv4 gets mapped to
-	 IPv6.  */
+         using `tcp6' as a protocol type, all connections will be
+         mapped to IPv6, and with `tcp6only', IPv4 gets mapped to
+         IPv6.  */
       int val = sep->se_v4mapped ? 0 : 1;
       if (setsockopt (sep->se_fd, IPPROTO_IPV6, IPV6_V6ONLY,
 		      (char *) &val, sizeof (val)) < 0)
@@ -619,9 +618,7 @@ setup (struct servtab *sep)
 #endif
   if (strncmp (sep->se_proto, "tcp", 3) == 0 && (options & SO_DEBUG))
     {
-      if (setsockopt (sep->se_fd, SOL_SOCKET, SO_DEBUG,
-		      (char *) &on, sizeof (on)) < 0
-	  && errno != EACCES)	/* Ignore insufficient permission.  */
+      if (setsockopt (sep->se_fd, SOL_SOCKET, SO_DEBUG, (char *) &on, sizeof (on)) < 0 && errno != EACCES)	/* Ignore insufficient permission.  */
 	syslog (LOG_ERR, "setsockopt (SO_DEBUG): %m");
     }
 
@@ -649,7 +646,7 @@ setup (struct servtab *sep)
 	    fprintf (stderr, "bind failed for %s %s/%s: %s\n",
 		     sep->se_node, sep->se_service, sep->se_proto,
 		     strerror (errno));
-	  syslog (LOG_ERR,"%s %s/%s: bind: %m",
+	  syslog (LOG_ERR, "%s %s/%s: bind: %m",
 		  sep->se_node, sep->se_service, sep->se_proto);
 	}
       else
@@ -657,7 +654,7 @@ setup (struct servtab *sep)
 	  if (debug)
 	    fprintf (stderr, "bind failed for %s/%s: %s\n",
 		     sep->se_service, sep->se_proto, strerror (errno));
-	  syslog (LOG_ERR,"%s/%s: bind: %m", sep->se_service, sep->se_proto);
+	  syslog (LOG_ERR, "%s/%s: bind: %m", sep->se_service, sep->se_proto);
 	}
       close (sep->se_fd);
       sep->se_fd = -1;
@@ -782,7 +779,7 @@ enter (struct servtab *cp)
   dupstr (&sep->se_user);
   dupstr (&sep->se_group);
   dupstr (&sep->se_server);
-  dupmem ((void**)&sep->se_argv, sep->se_argc * sizeof (sep->se_argv[0]));
+  dupmem ((void **) &sep->se_argv, sep->se_argc * sizeof (sep->se_argv[0]));
   for (i = 0; i < sep->se_argc; i++)
     dupstr (&sep->se_argv[i]);
 
@@ -815,11 +812,10 @@ inetd_getaddrinfo (struct servtab *sep, int proto, struct addrinfo **result)
   if (sep->se_node
       && (strspn (sep->se_node, IPV4_NUMCHARS) == strlen (sep->se_node)
 	  || (strchr (sep->se_node, ':')
-	      && strspn (sep->se_node, IPV6_NUMCHARS)) ) )
+	      && strspn (sep->se_node, IPV6_NUMCHARS))))
     numeric_address = true;
-  else
-    if (debug && sep->se_node)
-      fprintf (stderr, "Resolving address: %s\n", sep->se_node);
+  else if (debug && sep->se_node)
+    fprintf (stderr, "Resolving address: %s\n", sep->se_node);
 #endif /* IPV6 */
 
   memset (&hints, 0, sizeof (hints));
@@ -864,8 +860,7 @@ expand_enter (struct servtab *sep)
 
   err = inetd_getaddrinfo (sep, proto->p_proto, &result);
 #if IPV6
-  if (err == EAI_ADDRFAMILY
-      && sep->se_family == AF_INET6 && sep->se_v4mapped)
+  if (err == EAI_ADDRFAMILY && sep->se_family == AF_INET6 && sep->se_v4mapped)
     {
       /* Fall back to IPv4 silently.  */
       sep->se_family = AF_INET;
@@ -913,8 +908,8 @@ expand_enter (struct servtab *sep)
 
   return 0;
 }
-
 
+
 /* Configuration parser */
 char *global_serv_node;
 char *serv_node;
@@ -1015,9 +1010,9 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	  else if (n == 0)
 	    continue;
 
-	  if (linebuf[n-1] == '\n')
-	    linebuf[n-1] = 0;
-	  ++ *line;
+	  if (linebuf[n - 1] == '\n')
+	    linebuf[n - 1] = 0;
+	  ++*line;
 	}
       while (*linebuf == '#' || *linebuf == 0);
 
@@ -1045,7 +1040,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
       node = argv[INETD_SERVICE];
       service = strrchr (node, ':');
       if (!service)
-        {
+	{
 	  if (global_serv_node)
 	    {
 	      node = global_serv_node;
@@ -1053,21 +1048,21 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	      serv_node_offset = 0;
 	    }
 	  else
-	      node = NULL;
+	    node = NULL;
 
 	  service = argv[INETD_SERVICE];
-        }
+	}
       else
-        {
-          *service++ = 0;
-          if (strcmp (node, "*") == 0)
-            node = NULL;
+	{
+	  *service++ = 0;
+	  if (strcmp (node, "*") == 0)
+	    node = NULL;
 	  else
 	    {
 	      serv_node = newstr (node);
 	      serv_node_offset = 0;
 	    }
-        }
+	}
 
       if (strncmp (service, TCPMUX_TOKEN, MUX_LEN) == 0)
 	{
@@ -1120,7 +1115,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	      sep->se_v4mapped = 0;
 	      /* Check for tcp6only and udp6only.  */
 	      if (strcmp (&sep->se_proto[3], "6only") == 0)
-	        sep->se_v4mapped = 0;
+		sep->se_v4mapped = 0;
 	    }
 	  else if (sep->se_proto[3] == '4')
 	    {
@@ -1141,7 +1136,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
       {
 	char *p, *q;
 
-	p = strchr(argv[INETD_WAIT], '.');
+	p = strchr (argv[INETD_WAIT], '.');
 	if (p)
 	  *p++ = 0;
 	if (strcmp (argv[INETD_WAIT], "wait") == 0)
@@ -1155,7 +1150,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	  }
 	if (p)
 	  {
-	    sep->se_max = strtoul(p, &q, 10);
+	    sep->se_max = strtoul (p, &q, 10);
 	    if (*q)
 	      syslog (LOG_WARNING, "%s:%lu: invalid number (%s)",
 		      file, (unsigned long) *line, p);
@@ -1244,7 +1239,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	    argv0++;
 	  else
 	    argv0 = sep->se_server;
-          sep->se_argv[0] = newstr (argv0);
+	  sep->se_argv[0] = newstr (argv0);
 	}
 
       sep->se_argv[i] = NULL;
@@ -1373,7 +1368,8 @@ fix_tcpmux (void)
 	  /* Should not happen */
 	  freeconfig (&serv);
 	  if (debug)
-	    fprintf (stderr, "INTERNAL ERROR: could not find tcpmux built-in");
+	    fprintf (stderr,
+		     "INTERNAL ERROR: could not find tcpmux built-in");
 	  syslog (LOG_ERR, "INTERNAL ERROR: could not find tcpmux built-in");
 	  return;
 	}
@@ -1460,9 +1456,9 @@ config (int signo)
 
   fix_tcpmux ();
 }
-
 
 
+
 void
 set_proc_title (char *a, int s)
 {
@@ -1865,8 +1861,8 @@ prepenv (int ctrl, struct sockaddr *sa_client, socklen_t sa_len)
   else
     {
       ret = getnameinfo ((struct sockaddr *) &sa_server, len,
-			  ip, sizeof (ip), str, sizeof (str),
-			  NI_NUMERICHOST | NI_NUMERICSERV);
+			 ip, sizeof (ip), str, sizeof (str),
+			 NI_NUMERICHOST | NI_NUMERICSERV);
       if (ret == 0)
 	{
 	  if (setenv ("TCPLOCALIP", ip, 1) < 0)
@@ -1883,7 +1879,7 @@ prepenv (int ctrl, struct sockaddr *sa_client, socklen_t sa_len)
       if (resolve_option)
 	{
 	  ret = getnameinfo ((struct sockaddr *) &sa_server, len,
-			      ip, sizeof (ip), NULL, 0, 0);
+			     ip, sizeof (ip), NULL, 0, 0);
 	  if (ret != 0)
 	    syslog (LOG_WARNING, "getnameinfo: %s", gai_strerror (ret));
 	  else if (setenv ("TCPLOCALHOST", ip, 1) < 0)
@@ -1892,7 +1888,7 @@ prepenv (int ctrl, struct sockaddr *sa_client, socklen_t sa_len)
     }
 
   ret = getnameinfo (sa_client, sa_len, ip, sizeof (ip), str, sizeof (str),
-		      NI_NUMERICHOST | NI_NUMERICSERV);
+		     NI_NUMERICHOST | NI_NUMERICSERV);
   if (ret == 0)
     {
       if (setenv ("TCPREMOTEIP", ip, 1) < 0)
@@ -1917,9 +1913,9 @@ prepenv (int ctrl, struct sockaddr *sa_client, socklen_t sa_len)
   else
     syslog (LOG_WARNING, "getnameinfo: %s", gai_strerror (ret));
 }
-
 
 
+
 int
 main (int argc, char *argv[], char *envp[])
 {
@@ -1973,19 +1969,18 @@ main (int argc, char *argv[], char *envp[])
   openlog ("inetd", LOG_PID | LOG_NOWAIT, LOG_DAEMON);
 
   if (pidfile_option)
-  {
-    FILE *fp = fopen (pid_file, "w");
-    if (fp != NULL)
-      {
-	if (debug)
-	  fprintf(stderr, "Using pid-file at \"%s\".\n", pid_file);
-	fprintf (fp, "%d\n", (int) getpid ());
-	fclose (fp);
-      }
-    else
-      syslog (LOG_CRIT, "can't open %s: %s\n", pid_file,
-	      strerror (errno));
-  }
+    {
+      FILE *fp = fopen (pid_file, "w");
+      if (fp != NULL)
+	{
+	  if (debug)
+	    fprintf (stderr, "Using pid-file at \"%s\".\n", pid_file);
+	  fprintf (fp, "%d\n", (int) getpid ());
+	  fclose (fp);
+	}
+      else
+	syslog (LOG_CRIT, "can't open %s: %s\n", pid_file, strerror (errno));
+    }
 
   signal_set_handler (SIGALRM, retry);
   config (0);
@@ -2083,7 +2078,7 @@ main (int argc, char *argv[], char *envp[])
 				"%s/%s server failing (looping), service terminated",
 				sep->se_service, sep->se_proto);
 			close_sep (sep);
-			if (! sep->se_wait && sep->se_socktype == SOCK_STREAM)
+			if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
 			  close (ctrl);
 			signal_unblock (NULL);
 			if (!timingout)
