@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software
-  Foundation, Inc.
+  Copyright (C) 1995-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -68,15 +66,19 @@
  * of time
  */
 int
-ctl_transact (struct in_addr target, CTL_MSG msg, int type, CTL_RESPONSE * rp)
+ctl_transact (struct in_addr target, CTL_MSG msg, int type, CTL_RESPONSE *rp)
 {
-  int nready, cc;
+  int nready = 0, cc;
   fd_set read_mask, ctl_mask;
   struct timeval wait;
 
   msg.type = type;
+  daemon_addr.sin_family = AF_INET;
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
+  daemon_addr.sin_len = sizeof (daemon_addr);
+#endif
   daemon_addr.sin_addr = target;
-  daemon_addr.sin_port = daemon_port;
+  daemon_addr.sin_port = htons (daemon_port);
   FD_ZERO (&ctl_mask);
   FD_SET (ctl_sockt, &ctl_mask);
 
