@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software
-  Foundation, Inc.
+  Copyright (C) 1995-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -61,6 +59,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <setjmp.h>
+#include <attribute.h>
 #include "talk_ctl.h"
 #include "talk.h"
 
@@ -115,7 +114,7 @@ announce_invite (void)
  * Routine called on interupt to re-invite the callee
  */
 void
-re_invite (int sig)
+re_invite (int sig MAYBE_UNUSED)
 {
 
   message ("Ringing your party again");
@@ -194,6 +193,10 @@ send_delete (void)
    * and don't wait for an answer
    */
   msg.id_num = htonl (remote_id);
+  daemon_addr.sin_family = AF_INET;
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
+  daemon_addr.sin_len = sizeof (daemon_addr);
+#endif
   daemon_addr.sin_addr = his_machine_addr;
   if (sendto (ctl_sockt, (const char *) &msg, sizeof (msg), 0,
 	      (struct sockaddr *) &daemon_addr,
