@@ -1,7 +1,5 @@
 /*
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
-  2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Free
-  Software Foundation, Inc.
+  Copyright (C) 2002-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -53,7 +51,7 @@ startslave (char *host, int autologin, char *autoname)
       exit (EXIT_FAILURE);
     }
 #else /* !AUTHENTICATION */
-  (void) autoname;	/* Silence warnings.  */
+  (void) autoname;		/* Silence warnings.  */
 #endif
 
   pid = forkpty (&master, line, NULL, NULL);
@@ -116,7 +114,7 @@ start_login (char *host, int autologin, char *name)
   int argc;
   char **argv;
 
-  (void) host;		/* Silence warnings.  Diagnostic use?  */
+  (void) host;			/* Silence warnings.  Diagnostic use?  */
   (void) autologin;
   (void) name;
 
