@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
+  Copyright (C) 1998-2023 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -40,7 +39,7 @@
 #define obstack_chunk_free free
 #include <obstack.h>
 #include <error.h>
-#include <unused-parameter.h>
+#include <attribute.h>
 #include <libinetutils.h>
 
 /* Application-specific */
@@ -63,46 +62,45 @@ int nopar = 0;
 
 static struct argp_option ripe_argp_options[] = {
 #define GRP 10
-  { NULL, 'a', NULL, 0,
-    "search all databases", GRP },
-  { NULL, 'F', NULL, 0,
-    "fast raw output (implies -r)", GRP },
-  { NULL, 'g', "SOURCE:FIRST-LAST", 0,
-    "find updates from SOURCE from serial FIRST to LAST", GRP },
-  { NULL, 'i', "ATTR[,ATTR]...", 0,
-    "do an inverse lookup for specified ATTRibutes", GRP },
-  { NULL, 'l', NULL, 0,
-    "one level less specific lookup (RPSL only)", GRP },
-  { NULL, 'L', NULL, 0,
-    "find all Less specific matches", GRP },
-  { NULL, 'M', NULL, 0,
-    "find all More specific matches", GRP },
-  { NULL, 'm', NULL, 0,
-    "find first level more specific matches", GRP },
-  { NULL, 'r', NULL, 0,
-    "turn off recursive lookups", GRP },
-  { NULL, 'R', NULL, 0,
-    "force to show local copy of the domain object even "
-    "if it contains referral", GRP },
-  { NULL, 'S', NULL, 0,
-    "tell server to leave out syntactic sugar", GRP },
-  { NULL, 's', "SOURCE[,SOURCE]...", 0,
-    "search the database from SOURCE", GRP },
-  { NULL, 'T', "TYPE[,TYPE]...", 0,
-    "only look for objects of TYPE", GRP },
-  { NULL, 'q', "version|sources", 0,
-    "query specified server info (RPSL only)", GRP },
-  { NULL, 't', "TYPE", 0,
-    "requests template for object of TYPE ('all' for a list)", GRP },
-  { NULL, 'x', NULL, 0,
-    "exact match only (RPSL only)", GRP },
+  {NULL, 'a', NULL, 0,
+   "search all databases", GRP},
+  {NULL, 'F', NULL, 0,
+   "fast raw output (implies -r)", GRP},
+  {NULL, 'g', "SOURCE:FIRST-LAST", 0,
+   "find updates from SOURCE from serial FIRST to LAST", GRP},
+  {NULL, 'i', "ATTR[,ATTR]...", 0,
+   "do an inverse lookup for specified ATTRibutes", GRP},
+  {NULL, 'l', NULL, 0,
+   "one level less specific lookup (RPSL only)", GRP},
+  {NULL, 'L', NULL, 0,
+   "find all Less specific matches", GRP},
+  {NULL, 'M', NULL, 0,
+   "find all More specific matches", GRP},
+  {NULL, 'm', NULL, 0,
+   "find first level more specific matches", GRP},
+  {NULL, 'r', NULL, 0,
+   "turn off recursive lookups", GRP},
+  {NULL, 'R', NULL, 0,
+   "force to show local copy of the domain object even "
+   "if it contains referral", GRP},
+  {NULL, 'S', NULL, 0,
+   "tell server to leave out syntactic sugar", GRP},
+  {NULL, 's', "SOURCE[,SOURCE]...", 0,
+   "search the database from SOURCE", GRP},
+  {NULL, 'T', "TYPE[,TYPE]...", 0,
+   "only look for objects of TYPE", GRP},
+  {NULL, 'q', "version|sources", 0,
+   "query specified server info (RPSL only)", GRP},
+  {NULL, 't', "TYPE", 0,
+   "requests template for object of TYPE ('all' for a list)", GRP},
+  {NULL, 'x', NULL, 0,
+   "exact match only (RPSL only)", GRP},
 #undef GRP
-  { NULL, 0, NULL, 0, NULL, 0 }
+  {NULL, 0, NULL, 0, NULL, 0}
 };
 
 static error_t
-ripe_argp_parser (int key, char *arg,
-		  struct argp_state *state _GL_UNUSED_PARAMETER)
+ripe_argp_parser (int key, char *arg, struct argp_state *state MAYBE_UNUSED)
 {
   if (key > 0 && (unsigned) key < 128)
     {
@@ -125,22 +123,22 @@ static struct argp ripe_argp =
   { ripe_argp_options, ripe_argp_parser, NULL, NULL, NULL, NULL, NULL };
 
 static struct argp_option gwhois_argp_options[] = {
-#define GRP 10
-  { "verbose", 'V', NULL, 0,
-    "explain what is being done", GRP },
-  { "server", 'h', "HOST", 0,
-    "connect to server HOST", GRP},
-  { "port", 'p', "PORT", 0,
-    "connect to PORT", GRP },
-  { NULL, 'H', NULL, 0,
-    "hide legal disclaimers", GRP },
+#define GRP 20
+  {NULL, 0, NULL, 0, "General options", GRP},
+  {"verbose", 'V', NULL, 0,
+   "explain what is being done", GRP},
+  {"server", 'h', "HOST", 0,
+   "connect to server HOST", GRP},
+  {"port", 'p', "PORT", 0,
+   "connect to PORT", GRP},
+  {NULL, 'H', NULL, 0,
+   "hide legal disclaimers", GRP},
 #undef GRP
-  { NULL, 0, NULL, 0, NULL, 0 }
+  {NULL, 0, NULL, 0, NULL, 0}
 };
 
 static error_t
-gwhois_argp_parser (int key, char *arg,
-		    struct argp_state *state _GL_UNUSED_PARAMETER)
+gwhois_argp_parser (int key, char *arg, struct argp_state *state MAYBE_UNUSED)
 {
   char *p, *q;
 
@@ -155,7 +153,7 @@ gwhois_argp_parser (int key, char *arg,
       break;
 
     case 'H':
-      hide_discl = 0;	/* enable disclaimers hiding */
+      hide_discl = 0;		/* enable disclaimers hiding */
       break;
 
     case 'p':
@@ -173,12 +171,11 @@ gwhois_argp_parser (int key, char *arg,
 }
 
 struct argp_child gwhois_argp_children[] = {
-  { &ripe_argp,
-    0,
-    "RIPE-specific options",
-    0
-    },
-  { NULL, 0, NULL, 0 }
+  {&ripe_argp,
+   0,
+   "RIPE-specific options",
+   0},
+  {NULL, 0, NULL, 0}
 };
 
 static struct argp gwhois_argp = {
@@ -191,11 +188,10 @@ static struct argp gwhois_argp = {
   NULL
 };
 
-const char *program_authors[] =
-  {
-    "Marco d'Itri",
-    NULL
-  };
+const char *program_authors[] = {
+  "Marco d'Itri",
+  NULL
+};
 
 int
 main (int argc, char *argv[])
@@ -264,6 +260,7 @@ main (int argc, char *argv[])
 	case 1:
 	  puts (_("This TLD has no whois server, but you can access the "
 		  "whois database at"));
+	  /* FALLTHROUGH */
 	case 2:
 	  puts (server + 1);
 	  exit (EXIT_SUCCESS);
@@ -300,29 +297,22 @@ whichwhois (const char *s)
   unsigned long ip;
   unsigned int i;
 
+  /* Step over any single glyph preamble, as it must be some
+   * kind of informed instructions for an anticipated server.
+   */
+  while ((*s != '\0') && (*(s + 1) != '\0') && (*(s + 1) == ' '))
+    s += 2;
+
   /* -v or -t has been used */
   if (*s == '\0')
     return "whois.ripe.net";
 
-  /* IPv6 address */
-  if (strchr (s, ':'))
-    {
-      if (strncasecmp (s, "2001:2", 6) == 0)	/* XXX ugly hack! */
-	return "whois.apnic.net";
-      if (strncasecmp (s, "2001:4", 6) == 0)
-	return "whois.arin.net";
-      if (strncasecmp (s, "2001:6", 6) == 0)
-	return "whois.ripe.net";
-      /* if (strncasecmp(s, "3ffe", 4) == 0) */
-      return "whois.6bone.net";
-    }
-
   /* email address */
   if (strchr (s, '@'))
     return "";
 
-  /* no dot and no hyphen means it's a NSI NIC handle or ASN (?) */
-  if (!strpbrk (s, ".-"))
+  /* no dot, no colon, and no hyphen means it's an NSI NIC handle or ASN (?) */
+  if (!strpbrk (s, ".-:"))
     {
       const char *p;
 
@@ -338,6 +328,31 @@ whichwhois (const char *s)
 	return "";
     }
 
+  /* IPv6 address */
+  if (strchr (s, ':'))
+    {
+      int a = 0, b = 0;
+
+      /* Only the first two double-octets are ever scanned.
+       * At least one is required.
+       */
+      if (sscanf (s, "%x:%x:", &a, &b) == 0)
+	return "";
+
+      /* Mask off any non-significant bits and generate prefix.  */
+      ip = (a & 0xffff) << 16 | (b & 0xffff);
+
+      for (i = 0; ip6_assign[i].serv; i++)
+	if ((ip & ip6_assign[i].mask) == ip6_assign[i].net)
+	  return ip6_assign[i].serv;
+
+      if (verb)
+	puts (_("Delegation server of this address is not known.\n"
+		"The fallback is ARIN, hoping for success."));
+
+      return "whois.arin.net";
+    }
+
   /* smells like an IP? */
   if ((ip = myinet_aton (s)))
     {
@@ -380,7 +395,7 @@ whereas (int asn, struct as_del aslist[])
 {
   int i;
 
-  if (asn > 16383)
+  if (asn > 398287)
     puts (_("Unknown AS number. Please upgrade this program."));
   for (i = 0; aslist[i].serv; i++)
     if (asn >= aslist[i].first && asn <= aslist[i].last)
@@ -389,7 +404,7 @@ whereas (int asn, struct as_del aslist[])
 }
 
 int
-is_ripe_server (const char * const *srvtab, const char *name)
+is_ripe_server (const char *const *srvtab, const char *name)
 {
   struct in_addr addr;
   int isip = 0;
@@ -407,7 +422,7 @@ is_ripe_server (const char * const *srvtab, const char *name)
 	{
 	  char **pa;
 	  for (pa = hp->h_addr_list; *pa; pa++)
-	    if (*(unsigned long*)*pa == addr.s_addr)
+	    if (*(unsigned long *) *pa == addr.s_addr)
 	      return 1;
 	}
     }
@@ -425,7 +440,7 @@ queryformat (const char *server, const char *flags, const char *query)
   *buf = '\0';
 
   isripe = is_ripe_server (ripe_servers, server)
-            || is_ripe_server (ripe_servers_old, server);
+    || is_ripe_server (ripe_servers_old, server);
   if (isripe)
     strcat (buf, "-V" IDSTRING " ");
 
@@ -440,7 +455,7 @@ queryformat (const char *server, const char *flags, const char *query)
       (strcmp (server, "whois.arin.net") == 0 ||
        strcmp (server, "whois.nic.mil") == 0) &&
       strncasecmp (query, "AS", 2) == 0 && query[2] >= '0' && query[2] <= '9')
-    sprintf (buf, "AS %s", query + 2);	/* fix query for ARIN */
+    sprintf (buf, "a %s", query + 2);	/* fix query for ARIN */
   else if (!isripe && strcmp (server, "whois.corenic.net") == 0)
     sprintf (buf, "--machine %s", query);	/* machine readable output */
   else if (!isripe && strcmp (server, "whois.ncst.ernet.in") == 0 &&
@@ -469,15 +484,14 @@ do_query (const int sock, const char *query)
   fi = fdopen (sock, "r");
   if (write (sock, query, strlen (query)) < 0)
     err_sys ("write");
-  if (shutdown (sock, 1) < 0)
-    err_sys ("shutdown");
+
   while (fgets (buf, 200, fi))
     {				/* XXX errors? */
       if (hide == 1)
 	{
 	  if (strncmp (buf, hide_strings[i + 1], strlen (hide_strings[i + 1]))
 	      == 0)
-	    hide = 2;		/* stop hiding */
+	    hide = 0;		/* go into testing mode */
 	  continue;		/* hide this line */
 	}
       if (hide == 0)
@@ -540,6 +554,7 @@ query_crsnic (const int sock, const char *query)
   fi = fdopen (sock, "r");
   if (write (sock, temp, strlen (temp)) < 0)
     err_sys ("write");
+
   while (fgets (buf, 100, fi))
     {
       /* If there are multiple matches only the server of the first record
@@ -574,8 +589,14 @@ openconn (const char *server, const char *port)
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
 
+  /* The well-known service name is no longer universally acknowledged,
+   * because IANA has tampered with aliasing, making `Who' and `Is'
+   * into legal and functional aliases.
+   */
   if ((i = getaddrinfo (server, port ? port : "whois", &hints, &res)) != 0)
-    err_quit ("getaddrinfo: %s", gai_strerror (i));
+    if ((i =
+	 getaddrinfo (server, port ? port : "nicname", &hints, &res)) != 0)
+      err_quit ("getaddrinfo: %s", gai_strerror (i));
 
   for (ressave = res; res; res = res->ai_next)
     {
